@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2007, 2009 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009, 2010 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -137,7 +137,7 @@ class Answer(object):
         elif attr == 'rdtype':
             return self.rrset.rdtype
         else:
-            raise AttributeError, attr
+            raise AttributeError(attr)
 
     def __len__(self):
         return len(self.rrset)
@@ -169,7 +169,7 @@ class Cache(object):
     since the epoch.)
     @type next_cleaning: float
     """
-    
+
     def __init__(self, cleaning_interval=300.0):
         """Initialize a DNS cache.
 
@@ -177,14 +177,14 @@ class Cache(object):
         cleanings.  The default is 300.0
         @type cleaning_interval: float.
         """
-        
+
         self.data = {}
         self.cleaning_interval = cleaning_interval
         self.next_cleaning = time.time() + self.cleaning_interval
 
     def maybe_clean(self):
         """Clean the cache if it's time to do so."""
-        
+
         now = time.time()
         if self.next_cleaning <= now:
             keys_to_delete = []
@@ -195,7 +195,7 @@ class Cache(object):
                 del self.data[k]
             now = time.time()
             self.next_cleaning = now + self.cleaning_interval
-            
+
     def get(self, key):
         """Get the answer associated with I{key}.  Returns None if
         no answer is cached for the key.
@@ -204,7 +204,7 @@ class Cache(object):
         query name, rdtype, and rdclass.
         @rtype: dns.resolver.Answer object or None
         """
-        
+
         self.maybe_clean()
         v = self.data.get(key)
         if v is None or v.expiration <= time.time():
@@ -219,7 +219,7 @@ class Cache(object):
         @param value: The answer being cached
         @type value: dns.resolver.Answer object
         """
-        
+
         self.maybe_clean()
         self.data[key] = value
 
@@ -232,7 +232,7 @@ class Cache(object):
         @param key: the key to flush
         @type key: (dns.name.Name, int, int) tuple or None
         """
-        
+
         if not key is None:
             if self.data.has_key(key):
                 del self.data[key]
@@ -265,6 +265,9 @@ class Resolver(object):
     @type keyring: dict
     @ivar keyname: The TSIG keyname to use.  The default is None.
     @type keyname: dns.name.Name object
+    @ivar keyalgorithm: The TSIG key algorithm to use.  The default is
+    dns.tsig.default_algorithm.
+    @type keyalgorithm: string
     @ivar edns: The EDNS level to use.  The default is -1, no Edns.
     @type edns: int
     @ivar ednsflags: The EDNS flags
@@ -307,6 +310,7 @@ class Resolver(object):
         self.lifetime = 30.0
         self.keyring = None
         self.keyname = None
+        self.keyalgorithm = dns.tsig.default_algorithm
         self.edns = -1
         self.ednsflags = 0
         self.payload = 0
@@ -545,7 +549,7 @@ class Resolver(object):
         of the appropriate type, or strings that can be converted into objects
         of the appropriate type.  E.g. For I{rdtype} the integer 2 and the
         the string 'NS' both mean to query for records with DNS rdata type NS.
-        
+
         @param qname: the query name
         @type qname: dns.name.Name object or string
         @param rdtype: the query type
@@ -562,12 +566,12 @@ class Resolver(object):
         @raises NoAnswer: the response did not contain an answer
         @raises NoNameservers: no non-broken nameservers are available to
         answer the question."""
-        
+
         if isinstance(qname, (str, unicode)):
             qname = dns.name.from_text(qname, None)
-        if isinstance(rdtype, str):
+        if isinstance(rdtype, (str, unicode)):
             rdtype = dns.rdatatype.from_text(rdtype)
-        if isinstance(rdclass, str):
+        if isinstance(rdclass, (str, unicode)):
             rdclass = dns.rdataclass.from_text(rdclass)
         qnames_to_try = []
         if qname.is_absolute():
@@ -589,7 +593,8 @@ class Resolver(object):
                     return answer
             request = dns.message.make_query(qname, rdtype, rdclass)
             if not self.keyname is None:
-                request.use_tsig(self.keyring, self.keyname)
+                request.use_tsig(self.keyring, self.keyname,
+                                 algorithm=self.keyalgorithm)
             request.use_edns(self.edns, self.ednsflags, self.payload)
             response = None
             #
@@ -670,7 +675,8 @@ class Resolver(object):
             self.cache.put((qname, rdtype, rdclass), answer)
         return answer
 
-    def use_tsig(self, keyring, keyname=None):
+    def use_tsig(self, keyring, keyname=None,
+                 algorithm=dns.tsig.default_algorithm):
         """Add a TSIG signature to the query.
 
         @param keyring: The TSIG keyring to use; defaults to None.
@@ -680,12 +686,16 @@ class Resolver(object):
         but a keyname is not, then the key used will be the first key in the
         keyring.  Note that the order of keys in a dictionary is not defined,
         so applications should supply a keyname when a keyring is used, unless
-        they know the keyring contains only one key."""
+        they know the keyring contains only one key.
+        @param algorithm: The TSIG key algorithm to use.  The default
+        is dns.tsig.default_algorithm.
+        @type algorithm: string"""
         self.keyring = keyring
         if keyname is None:
             self.keyname = self.keyring.keys()[0]
         else:
             self.keyname = keyname
+        self.keyalgorithm = algorithm
 
     def use_edns(self, edns, ednsflags, payload):
         """Configure Edns.
@@ -740,13 +750,16 @@ def zone_for_name(name, rdclass=dns.rdataclass.IN, tcp=False, resolver=None):
     if resolver is None:
         resolver = get_default_resolver()
     if not name.is_absolute():
-        raise NotAbsolute, name
+        raise NotAbsolute(name)
     while 1:
         try:
             answer = resolver.query(name, dns.rdatatype.SOA, rdclass, tcp)
-            return name
+            if answer.rrset.name == name:
+                return name
+            # otherwise we were CNAMEd or DNAMEd and need to look higher
         except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
-            try:
-                name = name.parent()
-            except dns.name.NoParent:
-                raise NoRootSOA
+            pass
+        try:
+            name = name.parent()
+        except dns.name.NoParent:
+            raise NoRootSOA
